=== added directory 'nih'
=== added file 'nih/cnih.go'
--- nih/cnih.go	1970-01-01 00:00:00 +0000
+++ nih/cnih.go	2014-05-23 09:53:36 +0000
@@ -0,0 +1,27 @@
+package nih
+
+/*
+#cgo pkg-config: dbus-1 libnih libnih-dbus
+#include <stdlib.h>
+#include <nih/alloc.h>
+#include <libnih-dbus.h>
+
+char *cuote (const char *id) {
+    return nih_dbus_path (NULL, "", id, NULL);
+}
+*/
+import "C"
+
+import (
+	"unsafe"
+)
+
+func cuote(s []byte) string {
+	cs := C.CString(string(s))
+	defer C.free(unsafe.Pointer(cs))
+
+	cq := C.cuote(cs)
+	defer C.nih_free(unsafe.Pointer(cq))
+
+	return C.GoString(cq)[1:]
+}

=== added file 'nih/nih.go'
--- nih/nih.go	1970-01-01 00:00:00 +0000
+++ nih/nih.go	2014-05-23 09:53:36 +0000
@@ -0,0 +1,64 @@
+/*
+ Copyright 2013-2014 Canonical Ltd.
+
+ This program is free software: you can redistribute it and/or modify it
+ under the terms of the GNU General Public License version 3, as published
+ by the Free Software Foundation.
+
+ This program is distributed in the hope that it will be useful, but
+ WITHOUT ANY WARRANTY; without even the implied warranties of
+ MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
+ PURPOSE.  See the GNU General Public License for more details.
+
+ You should have received a copy of the GNU General Public License along
+ with this program.  If not, see <http://www.gnu.org/licenses/>.
+*/
+
+package nih
+
+import "strconv"
+
+func Quote(s []byte) []byte {
+	if len(s) == 0 {
+		return []byte{'_'}
+	}
+	out := make([]byte, 0, 2*len(s))
+	for _, c := range s {
+		if ('0' <= c && c <= '9') || ('a' <= c && c <= 'z') || ('A' <= c && c <= 'Z') {
+			out = append(out, c)
+		} else {
+			if c < 16 {
+				out = append(out, '_', '0')
+			} else {
+				out = append(out, '_')
+			}
+			out = strconv.AppendUint(out, uint64(c), 16)
+		}
+	}
+
+	return out
+}
+
+func Unquote(s []byte) []byte {
+	out := make([]byte, 0, len(s))
+	is_quote := false
+
+	for i := 0; i < len(s); i++ {
+		if is_quote {
+			num, err := strconv.ParseUint(string(s[i:i+2]), 16, 8)
+			if err == nil {
+				out = append(out, byte(num))
+			}
+			is_quote = false
+			i++
+		} else {
+			if s[i] == '_' {
+				is_quote = true
+			} else {
+				out = append(out, s[i])
+			}
+		}
+	}
+
+	return out
+}

=== added file 'nih/nih_test.go'
--- nih/nih_test.go	1970-01-01 00:00:00 +0000
+++ nih/nih_test.go	2014-05-23 09:53:36 +0000
@@ -0,0 +1,45 @@
+/*
+ Copyright 2013-2014 Canonical Ltd.
+
+ This program is free software: you can redistribute it and/or modify it
+ under the terms of the GNU General Public License version 3, as published
+ by the Free Software Foundation.
+
+ This program is distributed in the hope that it will be useful, but
+ WITHOUT ANY WARRANTY; without even the implied warranties of
+ MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
+ PURPOSE.  See the GNU General Public License for more details.
+
+ You should have received a copy of the GNU General Public License along
+ with this program.  If not, see <http://www.gnu.org/licenses/>.
+*/
+
+package nih
+
+import (
+	"testing"
+
+	. "launchpad.net/gocheck"
+)
+
+func TestNIH(t *testing.T) { TestingT(t) }
+
+type nihSuite struct{}
+
+var _ = Suite(&nihSuite{})
+
+func (ns *nihSuite) TestQuote(c *C) {
+	for i, s := range []struct {
+		raw    []byte
+		quoted []byte
+	}{
+		{[]byte("test"), []byte("test")},
+		{[]byte("foo/bar.baz"), []byte("foo_2fbar_2ebaz")},
+		{[]byte("test_thing"), []byte("test_5fthing")},
+		{nil, []byte{'_'}},
+	} {
+		c.Check(string(s.quoted), Equals, cuote(s.raw), Commentf("iter %d", i))
+		c.Check(Quote(s.raw), DeepEquals, s.quoted, Commentf("iter %d", i))
+		c.Check(string(Quote(s.raw)), Equals, cuote(s.raw), Commentf("iter %d", i))
+	}
+}

